@@ -1,8 +1,11 @@
 /* gxmessage - an xmessage clone using GTK2
  *
- * $Id: gxmessage.c,v 1.23 2006/01/23 03:40:07 trm Exp trm $
+ * $Id: gxmessage.c,v 1.25 2006/10/23 14:38:58 trm Exp trm $
  *
- * Copyright Timothy Musson 2001 - 2006, <trmusson@ihug.co.nz>
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006 Timothy Musson
+ *
+ * <trmusson@ihug.co.nz>
+ * http://homepages.ihug.co.nz/~trmusson/programs.html#gxmessage
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -49,7 +52,6 @@
 #define FNAME_ICON    PACKAGE_DATA_DIR G_DIR_SEPARATOR_S \
                       "pixmaps" G_DIR_SEPARATOR_S "gxmessage.png"
 
-
 typedef struct _Button Button;
 
 
@@ -372,11 +374,11 @@ cb_entry_activated (GtkWidget *widget, gpointer data)
 
 
 gboolean
-cb_timeout (gpointer data)
+cb_timeout (gint *timeout)
 {
 	static gint counter = 0;
 
-	if (++counter >= (gint)data) {
+	if (++counter >= *timeout) {
 		gx.exit_code = 0;
 		gtk_main_quit ();
 	}
@@ -582,12 +584,14 @@ window_create (void)
 	max_w = gdk_screen_width () * 0.7;
 	max_h = gdk_screen_height () * 0.7;
 
-	if (gx.message_len > 3000) {
+	/* Render dummy text, to get an idea of its size. This is slow when
+	 * there's a lot of text, so default to max_w and max_h in that case.
+	 */
+	if (gx.message_len > 20000) {
 		win_w = max_w;
 		win_h = max_h;
 	}
 	else {
-		/* render dummy text, to get an idea of its size */
 		GtkWidget *dummy = gtk_label_new (gx.message_text);
 		gtk_widget_modify_font (dummy,
 		    gtk_widget_get_style (message_widget)->font_desc);
@@ -627,7 +631,7 @@ window_create (void)
 	/* begin timeout */
 	if (gx.timeout != 0) {
 		gx.timeout_id = g_timeout_add (1000, (GtkFunction)cb_timeout,
-		                               (gpointer)gx.timeout);
+		                               &gx.timeout);
 	}
 }
 
